import Jama.Matrix;
import Jama.SingularValueDecomposition;

public class superimpose
{
	public static void main(String[] argv)
	{
		
		//sample usage of localSuperimpose function
		
	    double[][] pi1 = new double[10][3];
	    double[][] pi2 = new double[15][3];
	    
	    
	    // now let's fill in these two point sets with random coordinates
	    
	    for (int i=0;i<10;i++) // 10 points in first set
	       for (int j=0;j<3;j++)  // x,y,z coordinates
	       {
	       	   pi1[i][j] = Math.random();  
	       }
	    for (int i=0;i<15;i++) // 15 points in second set
	       for (int j=0;j<3;j++)  // x,y,z coordinates
	       {
	       	   pi2[i][j] = Math.random();
	       }
	       
	    // now let's superimpose them by using only their first 5 points.
	    int[] asc1 = new int[5];
	    int[] asc2 = new int[5];
	    
        for (int i=0;i<5;i++)
        {
        	asc1[i]=i;
        	asc2[i]=i;	    
        }
        
        // prepare storage for best rotation and translation vectors
        
        double[][] rot = new double[3][3];
        double[] trans = new double[3];
        
        double rmsd = localSuperImpose(pi1,pi2,rot,trans,asc1,asc2);
        System.out.println("RMSD 1 = "+rmsd);
        System.out.println("Best Rotation Matrix 1:");
	  	for (int i=0;i<3;i++)
	  	{
		  for (int j=0;j<3;j++)
			  System.out.print(rot[i][j]+"\t");
		  System.out.println();
	  	}
	  	System.out.println("Translation Vector 1:");
	  	for (int i=0;i<3;i++)
		  System.out.print(trans[i]+"\t");
	  	System.out.println();


	    // now let's superimpose them by using first 5 points from set 1 and last 5 point from set 2.
	    
        for (int i=0;i<5;i++)
        {
        	asc1[i]=i;
        	asc2[i]=10+i;	    
        }
        
        rmsd = localSuperImpose(pi1,pi2,rot,trans,asc1,asc2);
        System.out.println("RMSD 2 = "+rmsd);
        System.out.println("Best Rotation Matrix 2:");
	  	for (int i=0;i<3;i++)
	  	{
		  for (int j=0;j<3;j++)
			  System.out.print(rot[i][j]+"\t");
		  System.out.println();
	  	}
	  	System.out.println("Translation Vector 2:");
	  	for (int i=0;i<3;i++)
		  System.out.print(trans[i]+"\t");
	  	System.out.println();
	       
	}

	  

  // This function superimposes two point sets locally according to some association between them
  // points1 and points2 can be of different length
  // however asc1 and asc2 should be of same length
  // The association information is used as follows:
  //    points1[asc1[n]] is associated with points2[asc2[n]]
  public static double localSuperImpose(double[][] points1, double[][] points2, double[][] rvec, double[] tvec, int[] asc1, int [] asc2)
  {
	  int n;
	  int i,j,k;
	  
	  double minrmsd = 5000000.0f; // a very big number goes here
	  double rmsd;
	  
	  n = asc1.length;
	  
	  double[][] pi1;  // The list of 3-d points
	  double[][] pi2;  // The list of 3-d points
	  double[] p1; // The centroid of first point set
	  double[] p2; // The centroid of second point set
	  double[][] qi1; // The difference vector (coord - centroid) of first point set
	  double[][] qi2; // The difference vector (coord - centroid) of second point set
	  
	  double[][] H;  // qi1 * qi2(T)
	  
	  pi1 = new double[n][3];
	  pi2 = new double[n][3];
	  
	  p1 = new double[3];
	  p2 = new double[3];
	  qi1 = new double[n][3];
	  qi2 = new double[n][3];
	  H = new double[3][3];

	  for (i=0;i<n;i++)
	  {
  		  pi1[i][0]=points1[asc1[i]][0]; // x-coordinate
		  pi1[i][1]=points1[asc1[i]][1]; // y-coordinate
		  pi1[i][2]=points1[asc1[i]][2]; // z-coordinate

		  pi2[i][0]=points2[asc2[i]][0]; // x-coordinate
		  pi2[i][1]=points2[asc2[i]][1]; // y-coordinate
		  pi2[i][2]=points2[asc2[i]][2]; // z-coordinate
	  }
		  
	  for (j=0;j<3;j++)
	  {
		  p1[j] = 0.0;
		  p2[j] = 0.0;
      }
		  
	  // computation of the centroids
	  for (i=0;i<n;i++)
              for (j=0;j<3;j++)
              {
                  p1[j]+=pi1[i][j];
                  p2[j]+=pi2[i][j];
              }
       for (j=0;j<3;j++)
       {
           p1[j]=p1[j]/n;
           p2[j]=p2[j]/n;
       }
       // computation of the centroids complete
		  
	  for (i=0;i<n;i++)
              for (j=0;j<3;j++)
              {
                  qi1[i][j]=pi1[i][j]-p1[j];
                  qi2[i][j]=pi2[i][j]-p2[j];
              }
      // computation of the difference vectors complete
		 
	  for (i=0;i<3;i++)
              for (j=0;j<3;j++)
                  H[i][j]=0.0;
		  
	  for (i=0;i<3;i++)
              for (j=0;j<3;j++)
                  for (k=0;k<n;k++)
                      H[i][j]+=qi1[k][i]*qi2[k][j];
		  
		  
	  /* To debug the SVD computation
	  System.out.println("H Matrix");
	  for (i=0;i<3;i++)
	  {
		  for (j=0;j<3;j++)
			  System.out.print(H[i][j]+" ");
		  System.out.println();
	  }*/
		  
		  
	  Matrix Hmat = new Matrix(H);
	  SingularValueDecomposition svd = new SingularValueDecomposition(Hmat);
		  

	  for (i=0;i<3;i++)
		  for (j=0;j<3;j++)
			  rvec[i][j] = svd.getV().getArray()[i][0] * svd.getU().getArray()[j][0] +
                                       svd.getV().getArray()[i][1] * svd.getU().getArray()[j][1] +
                                       svd.getV().getArray()[i][2] * svd.getU().getArray()[j][2];
		  
	  /// application of Umeyama's result
		  
	  if ((svd.getV().det()*svd.getU().det())<0)
          {
              rvec[0][2]=-rvec[0][2];
              rvec[1][2]=-rvec[1][2];
              rvec[2][2]=-rvec[2][2];
          }
		  	
		  
		  
	  double det = (new Matrix(rvec)).det();
          if (det<0.0)
          {
              System.out.println("!!!!!!!!!!!!!!!!!!Unexpected result!!!!!!!!!!!!!!!!!!");
              System.exit(1);
          }
	  
      for (i=0;i<3;i++)
          tvec[i] = p2[i] - (rvec[i][0]*p1[0]+rvec[i][1]*p1[1]+rvec[i][2]*p1[2]);
		    
	  // Now compute RMSD (pi2 - pi1)
      // after tranforming pi1 to new coordinates
      // new pi1 = pi1*rvec + tvec
		  
	  double[][] newpi1 = new double[n][3];
	  for (i=0;i<n;i++)
              for (j=0;j<3;j++)
                  newpi1[i][j] = rvec[j][0]*pi1[i][0]+rvec[j][1]*pi1[i][1]+rvec[j][2]*pi1[i][2]+tvec[j];
		  	  
          double totalErr = 0.0;
          for (i=0;i<n;i++)
              totalErr += (pi2[i][0]-newpi1[i][0])*(pi2[i][0]-newpi1[i][0])+
                          (pi2[i][1]-newpi1[i][1])*(pi2[i][1]-newpi1[i][1])+
                          (pi2[i][2]-newpi1[i][2])*(pi2[i][2]-newpi1[i][2]);
		  
	  minrmsd = Math.sqrt(totalErr/n);								  
          //System.out.println("Minimum RMSD : "+minrmsd);

//	  System.out.println("determinant = "+(new Matrix(rvec)).det());
/*	  System.out.println("Best Rotation Matrix :");
	  for (i=0;i<3;i++)
	  {
		  for (j=0;j<3;j++)
			  System.out.print(rvec[i][j]+" ");
		  System.out.println();
	  }
	  System.out.println("Best Translation Vector");
	  for (i=0;i<3;i++)
		  System.out.print(tvec[i]+" ");
	  System.out.println();*/
	
	  return minrmsd;
  }     
}